@@ -1,12 +1,12 @@
 /* Solaris support needed only by C/C++ frontends.
-   Copyright (C) 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005 , 2007 Free Software Foundation, Inc.
    Contributed by CodeSourcery, LLC.
 
 This file is part of GCC.
 
 GCC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GCC is distributed in the hope that it will be useful,
@@ -15,9 +15,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GCC; see the file COPYING.  If not, write to
-the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.  */
+along with GCC; see the file COPYING3.  If not see
+<http://www.gnu.org/licenses/>.  */
 
 #include "config.h"
 #include "system.h"
@@ -88,8 +87,8 @@ solaris_pragma_align (cpp_reader *pfile ATTRIBUTE_UNUSED)
   enum cpp_ttype ttype;
   HOST_WIDE_INT low;
 
-  if (c_lex (&x) != CPP_NUMBER
-      || c_lex (&t) != CPP_OPEN_PAREN)
+  if (pragma_lex (&x) != CPP_NUMBER
+      || pragma_lex (&t) != CPP_OPEN_PAREN)
     {
       warning (0, "malformed %<#pragma align%>, ignoring");
       return;
@@ -104,7 +103,7 @@ solaris_pragma_align (cpp_reader *pfile ATTRIBUTE_UNUSED)
       return;
     }
 
-  ttype = c_lex (&t);
+  ttype = pragma_lex (&t);
   if (ttype != CPP_NAME)
     {
       warning (0, "malformed %<#pragma align%>, ignoring");
@@ -121,10 +120,10 @@ solaris_pragma_align (cpp_reader *pfile ATTRIBUTE_UNUSED)
 	solaris_pending_aligns = tree_cons (t, build_tree_list (NULL, x),
 					    solaris_pending_aligns);
 
-      ttype = c_lex (&t);
+      ttype = pragma_lex (&t);
       if (ttype == CPP_COMMA)
 	{
-	  ttype = c_lex (&t);
+	  ttype = pragma_lex (&t);
 	  if (ttype != CPP_NAME)
 	    {
 	      warning (0, "malformed %<#pragma align%>");
@@ -133,7 +132,7 @@ solaris_pragma_align (cpp_reader *pfile ATTRIBUTE_UNUSED)
 	}
       else if (ttype == CPP_CLOSE_PAREN)
 	{
-	  if (c_lex (&t) != CPP_EOF)
+	  if (pragma_lex (&t) != CPP_EOF)
 	    warning (0, "junk at end of %<#pragma align%>");
 	  return;
 	}
@@ -153,13 +152,13 @@ solaris_pragma_init (cpp_reader *pfile ATTRIBUTE_UNUSED)
   tree t;
   enum cpp_ttype ttype;
 
-  if (c_lex (&t) != CPP_OPEN_PAREN)
+  if (pragma_lex (&t) != CPP_OPEN_PAREN)
     {
       warning (0, "malformed %<#pragma init%>, ignoring");
       return;
     }
 
-  ttype = c_lex (&t);
+  ttype = pragma_lex (&t);
   if (ttype != CPP_NAME)
     {
       warning (0, "malformed %<#pragma init%>, ignoring");
@@ -179,10 +178,10 @@ solaris_pragma_init (cpp_reader *pfile ATTRIBUTE_UNUSED)
       else
 	solaris_pending_inits = tree_cons (t, NULL, solaris_pending_inits);
 
-      ttype = c_lex (&t);
+      ttype = pragma_lex (&t);
       if (ttype == CPP_COMMA)
 	{
-	  ttype = c_lex (&t);
+	  ttype = pragma_lex (&t);
 	  if (ttype != CPP_NAME)
 	    {
 	      warning (0, "malformed %<#pragma init%>");
@@ -191,7 +190,7 @@ solaris_pragma_init (cpp_reader *pfile ATTRIBUTE_UNUSED)
 	}
       else if (ttype == CPP_CLOSE_PAREN)
 	{
-	  if (c_lex (&t) != CPP_EOF)
+	  if (pragma_lex (&t) != CPP_EOF)
 	    warning (0, "junk at end of %<#pragma init%>");
 	  return;
 	}
@@ -211,13 +210,13 @@ solaris_pragma_fini (cpp_reader *pfile ATTRIBUTE_UNUSED)
   tree t;
   enum cpp_ttype ttype;
 
-  if (c_lex (&t) != CPP_OPEN_PAREN)
+  if (pragma_lex (&t) != CPP_OPEN_PAREN)
     {
       warning (0, "malformed %<#pragma fini%>, ignoring");
       return;
     }
 
-  ttype = c_lex (&t);
+  ttype = pragma_lex (&t);
   if (ttype != CPP_NAME)
     {
       warning (0, "malformed %<#pragma fini%>, ignoring");
@@ -237,10 +236,10 @@ solaris_pragma_fini (cpp_reader *pfile ATTRIBUTE_UNUSED)
       else
 	solaris_pending_finis = tree_cons (t, NULL, solaris_pending_finis);
 
-      ttype = c_lex (&t);
+      ttype = pragma_lex (&t);
       if (ttype == CPP_COMMA)
 	{
-	  ttype = c_lex (&t);
+	  ttype = pragma_lex (&t);
 	  if (ttype != CPP_NAME)
 	    {
 	      warning (0, "malformed %<#pragma fini%>");
@@ -249,7 +248,7 @@ solaris_pragma_fini (cpp_reader *pfile ATTRIBUTE_UNUSED)
 	}
       else if (ttype == CPP_CLOSE_PAREN)
 	{
-	  if (c_lex (&t) != CPP_EOF)
+	  if (pragma_lex (&t) != CPP_EOF)
 	    warning (0, "junk at end of %<#pragma fini%>");
 	  return;
 	}
